# -*- coding: utf-8 -*-

# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: google/cloud/orgpolicy/v1/orgpolicy.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n)google/cloud/orgpolicy/v1/orgpolicy.proto\x12\x19google.cloud.orgpolicy.v1\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1cgoogle/api/annotations.proto"\x8b\x05\n\x06Policy\x12\x0f\n\x07version\x18\x01 \x01(\x05\x12\x12\n\nconstraint\x18\x02 \x01(\t\x12\x0c\n\x04\x65tag\x18\x03 \x01(\x0c\x12/\n\x0bupdate_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x43\n\x0blist_policy\x18\x05 \x01(\x0b\x32,.google.cloud.orgpolicy.v1.Policy.ListPolicyH\x00\x12I\n\x0e\x62oolean_policy\x18\x06 \x01(\x0b\x32/.google.cloud.orgpolicy.v1.Policy.BooleanPolicyH\x00\x12K\n\x0frestore_default\x18\x07 \x01(\x0b\x32\x30.google.cloud.orgpolicy.v1.Policy.RestoreDefaultH\x00\x1a\xfb\x01\n\nListPolicy\x12\x16\n\x0e\x61llowed_values\x18\x01 \x03(\t\x12\x15\n\rdenied_values\x18\x02 \x03(\t\x12J\n\nall_values\x18\x03 \x01(\x0e\x32\x36.google.cloud.orgpolicy.v1.Policy.ListPolicy.AllValues\x12\x17\n\x0fsuggested_value\x18\x04 \x01(\t\x12\x1b\n\x13inherit_from_parent\x18\x05 \x01(\x08"<\n\tAllValues\x12\x1a\n\x16\x41LL_VALUES_UNSPECIFIED\x10\x00\x12\t\n\x05\x41LLOW\x10\x01\x12\x08\n\x04\x44\x45NY\x10\x02\x1a!\n\rBooleanPolicy\x12\x10\n\x08\x65nforced\x18\x01 \x01(\x08\x1a\x10\n\x0eRestoreDefaultB\r\n\x0bpolicy_typeB\xcc\x01\n\x1d\x63om.google.cloud.orgpolicy.v1B\x0eOrgPolicyProtoP\x01ZBgoogle.golang.org/genproto/googleapis/cloud/orgpolicy/v1;orgpolicy\xaa\x02\x19Google.Cloud.OrgPolicy.V1\xca\x02\x19Google\\Cloud\\OrgPolicy\\V1\xea\x02\x1cGoogle::Cloud::OrgPolicy::V1b\x06proto3'
)


_POLICY = DESCRIPTOR.message_types_by_name["Policy"]
_POLICY_LISTPOLICY = _POLICY.nested_types_by_name["ListPolicy"]
_POLICY_BOOLEANPOLICY = _POLICY.nested_types_by_name["BooleanPolicy"]
_POLICY_RESTOREDEFAULT = _POLICY.nested_types_by_name["RestoreDefault"]
_POLICY_LISTPOLICY_ALLVALUES = _POLICY_LISTPOLICY.enum_types_by_name["AllValues"]
Policy = _reflection.GeneratedProtocolMessageType(
    "Policy",
    (_message.Message,),
    {
        "ListPolicy": _reflection.GeneratedProtocolMessageType(
            "ListPolicy",
            (_message.Message,),
            {
                "DESCRIPTOR": _POLICY_LISTPOLICY,
                "__module__": "google.cloud.orgpolicy.v1.orgpolicy_pb2"
                # @@protoc_insertion_point(class_scope:google.cloud.orgpolicy.v1.Policy.ListPolicy)
            },
        ),
        "BooleanPolicy": _reflection.GeneratedProtocolMessageType(
            "BooleanPolicy",
            (_message.Message,),
            {
                "DESCRIPTOR": _POLICY_BOOLEANPOLICY,
                "__module__": "google.cloud.orgpolicy.v1.orgpolicy_pb2"
                # @@protoc_insertion_point(class_scope:google.cloud.orgpolicy.v1.Policy.BooleanPolicy)
            },
        ),
        "RestoreDefault": _reflection.GeneratedProtocolMessageType(
            "RestoreDefault",
            (_message.Message,),
            {
                "DESCRIPTOR": _POLICY_RESTOREDEFAULT,
                "__module__": "google.cloud.orgpolicy.v1.orgpolicy_pb2"
                # @@protoc_insertion_point(class_scope:google.cloud.orgpolicy.v1.Policy.RestoreDefault)
            },
        ),
        "DESCRIPTOR": _POLICY,
        "__module__": "google.cloud.orgpolicy.v1.orgpolicy_pb2"
        # @@protoc_insertion_point(class_scope:google.cloud.orgpolicy.v1.Policy)
    },
)
_sym_db.RegisterMessage(Policy)
_sym_db.RegisterMessage(Policy.ListPolicy)
_sym_db.RegisterMessage(Policy.BooleanPolicy)
_sym_db.RegisterMessage(Policy.RestoreDefault)

if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    DESCRIPTOR._serialized_options = b"\n\035com.google.cloud.orgpolicy.v1B\016OrgPolicyProtoP\001ZBgoogle.golang.org/genproto/googleapis/cloud/orgpolicy/v1;orgpolicy\252\002\031Google.Cloud.OrgPolicy.V1\312\002\031Google\\Cloud\\OrgPolicy\\V1\352\002\034Google::Cloud::OrgPolicy::V1"
    _POLICY._serialized_start = 165
    _POLICY._serialized_end = 816
    _POLICY_LISTPOLICY._serialized_start = 497
    _POLICY_LISTPOLICY._serialized_end = 748
    _POLICY_LISTPOLICY_ALLVALUES._serialized_start = 688
    _POLICY_LISTPOLICY_ALLVALUES._serialized_end = 748
    _POLICY_BOOLEANPOLICY._serialized_start = 750
    _POLICY_BOOLEANPOLICY._serialized_end = 783
    _POLICY_RESTOREDEFAULT._serialized_start = 785
    _POLICY_RESTOREDEFAULT._serialized_end = 801
# @@protoc_insertion_point(module_scope)
